/*
    ESP TCP Server
    - listen for incoming TCP messages
    - respond to the client with data in JSON form

    by noiasca
    2024-03-07
*/

#include <WiFi.h>
#include <ArduinoJson.h>                       // Install via library manager. Use a Version 6.x.x 

#define TXT_BOARDID "10"                       // an ID for the board
const uint16_t serverPort = 1040;              // 1040

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>                                   // if you have a config file with your credentials

#ifndef STASSID
#define STASSID "your-ssid"                                // set your SSID
#define STAPSK  "your-password"                            // set your wifi password
#endif

const char* ssid =     STASSID;
const char* password = STAPSK;

WiFiServer tcpServer(serverPort);

// thats the handler for the TCP Server
void handleTcpServer() {
  WiFiClient tcpClient = tcpServer.available();  // when a client called this server we need client to answer
  if (tcpClient) {
    Serial.println("incomming TCP client has connected");
    // read request (= the incoming data):
    while (tcpClient.available()) {
      String line = tcpClient.readStringUntil('\r');
      Serial.print(line);
    }
    Serial.println();
    String output = "";
    output.reserve(1000);  // reserve enough space for the output data
    // prepare response
    // paste here the source code as provided from the ArduinoJSON Asistant
    // https://arduinojson.org/v6/assistant/
    // then modify the code to set the variables according to your needs:
    StaticJsonDocument<128> doc;

    doc["runtime"] = millis() / 1000;
    doc["temperature"] = random(0, 500) / 10.0;
    doc["humidity"] = random(100, 1000) / 10.0;
    doc["pressure"] = 1024.4;

    serializeJson(doc, output);
    // optional print the generated JSON locally:
    Serial.println(output);
    // send response
    tcpClient.print(output);
    delay(10);
    tcpClient.stop();
    Serial.println("TCP client stopped");
  }
}

void setup() {
  Serial.begin(115200);
  while (!Serial) {
    delay(100);
  }
  Serial.println(F("\nTCP Server JSON"));
  Serial.print  (__DATE__);
  Serial.print  (' ');
  Serial.println(__TIME__);

  char myhostname[8] = {"esp"};
  strcat(myhostname, TXT_BOARDID);
  WiFi.hostname(myhostname);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWiFi connected");
  Serial.print("IP address: "); Serial.println(WiFi.localIP());

  Serial.print("listening for TCP on port:"); Serial.println(serverPort);
  tcpServer.begin();
}

void loop() {
  handleTcpServer();
  // do other things in the loop non blocking.
  // E.g. read your sensor data
}
//
